@@ -248,71 +248,65 @@ pub fn render(line_style: &LineStyle, prefix: &str, tokens: &[StyledToken]) -> S
     return rendered;
 }
 
-/// Unhighlight rows that have too much highlighting.
-///
-/// Returns true if something was unhighlighted, false otherwise.
-pub fn unhighlight_noisy_rows(tokens: &mut [StyledToken]) -> bool {
-    fn maybe_unhighlight_row(row: &mut [StyledToken]) -> bool {
-        // Count highlighted and not highlighted tokens, ignoring leading whitespace
-        let mut in_indentation = true;
-        let mut highlighted_chars_count = 0;
-        let mut all_chars_count = 0;
-        let mut highlighted_tokens_count = 0;
-        let mut all_tokens_count = 0;
-        for token in row.iter() {
-            if in_indentation {
-                if token.is_whitespace() {
-                    // Ignore indentation when counting the highlighting percentage
-                    continue;
-                }
-                in_indentation = false;
-            }
+/// Unhighlight runs that have too much highlighting.
+pub fn denoise(tokens: &mut [StyledToken]) -> bool {
+    /// Unhighlight a run if:
+    /// - It contains only whitespace
+    /// - It contains any newline
+    /// - It isn't a newline marker ("⏎"), we always leave those
+    ///
+    /// Returns true if something was unhighlighted, false otherwise.
+    fn maybe_unhighlight_run(run: &mut [StyledToken]) -> bool {
+        let whitespace_only = run.iter().all(|token| token.is_whitespace());
+        let contains_newline = run.iter().any(|token| token.token == "\n");
+        let is_newline_marker = run.len() == 2 && run[0].token == "⏎" && run[1].token == "\n";
 
+        if is_newline_marker {
+            return false;
+        }
+
+        if !whitespace_only && !contains_newline {
+            return false;
+        }
+
+        let mut i_did_it = false;
+        for token in run {
             if token.style == Style::Highlighted {
-                highlighted_chars_count += token.token.chars().count();
-                highlighted_tokens_count += 1;
+                token.style = Style::Plain;
+                i_did_it = true;
             }
-            all_chars_count += token.token.chars().count();
-            all_tokens_count += 1;
         }
 
-        if all_chars_count == 0 {
-            return false;
-        }
-
-        let highlighted_chars_percentage = (100 * highlighted_chars_count) / all_chars_count;
-        let highlighted_tokens_percentage = (100 * highlighted_tokens_count) / all_tokens_count;
-        if highlighted_chars_percentage < 70 || highlighted_tokens_percentage < 50 {
-            // Little enough of the line is highlighted, this is fine
-            return false;
-        }
-
-        // Unhighlight the current row
-        for token in row.iter_mut() {
-            token.style = Style::Plain;
-        }
-        return true;
+        return i_did_it;
     }
 
-    let mut line_start_index = 0;
-    let mut changed = false;
-
+    let mut i_did_it = false;
+    let mut current_run_start = 0;
+    let mut is_in_run = false;
     for i in 0..tokens.len() {
-        let token = &tokens[i];
-        if token.token == "\n" {
-            // End of line, evaluate!
-            changed |= maybe_unhighlight_row(&mut tokens[line_start_index..i]);
-
-            // Reset for the next row
-            line_start_index = i + 1;
+        if tokens[i].style == Style::Highlighted {
+            if !is_in_run {
+                current_run_start = i;
+                is_in_run = true;
+            }
             continue;
         }
+
+        // Not in a run
+
+        if is_in_run {
+            // A run just ended
+            i_did_it |= maybe_unhighlight_run(&mut tokens[current_run_start..i]);
+            is_in_run = false;
+        }
     }
 
-    // Handle the last row
-    changed |= maybe_unhighlight_row(&mut tokens[line_start_index..]);
+    if is_in_run {
+        // A run was still in progress at the end
+        i_did_it |= maybe_unhighlight_run(&mut tokens[current_run_start..]);
+    }
 
-    return changed;
+    return i_did_it;
 }
 
 pub fn errorlight_trailing_whitespace(tokens: &mut [StyledToken]) {
@@ -350,30 +344,6 @@ pub fn errorlight_nonleading_tabs(tokens: &mut [StyledToken]) {
     }
 }
 
-/// Returns true if something was unhighlighted, false otherwise.
-pub fn unhighlight_leading_whitespace(tokens: &mut [StyledToken]) -> bool {
-    let mut i_did_it = false;
-    let mut in_leading = true;
-    for token in tokens.iter_mut() {
-        if token.token == "\n" {
-            in_leading = true;
-            continue;
-        }
-
-        if in_leading && token.is_whitespace() {
-            if token.style == Style::Highlighted {
-                token.style = Style::Plain;
-                i_did_it = true;
-            }
-            continue;
-        }
-
-        in_leading = false;
-    }
-
-    return i_did_it;
-}
-
 pub(crate) fn align_tabs(old: &mut [StyledToken], new: &mut [StyledToken]) {
     let old_tab_index_token = old.iter().position(|token| token.token == "\t");
     if old_tab_index_token.is_none() {
@@ -444,49 +414,15 @@ pub fn contextualize_unhighlighted_lines(tokens: &mut [StyledToken]) {
 
 /// Highlight single space between two highlighted tokens
 pub fn bridge_consecutive_highlighted_tokens(tokens: &mut [StyledToken]) {
-    enum FoundState {
-        Nothing,
-        HighlightedWord,
-        WordSpace,
-    }
-
-    let mut found_state = FoundState::Nothing;
-    let mut previous_token: Option<&mut StyledToken> = None;
-    for token in tokens.iter_mut() {
-        match found_state {
-            FoundState::Nothing => {
-                if token.style == Style::Highlighted {
-                    // Found "Monkey"
-                    found_state = FoundState::HighlightedWord;
-                }
-            }
-
-            FoundState::HighlightedWord => {
-                if token.token.len() == 1 {
-                    // Found "Monkey " (note trailing space)
-                    found_state = FoundState::WordSpace;
-                } else if token.style == Style::Highlighted {
-                    found_state = FoundState::HighlightedWord;
-                } else {
-                    found_state = FoundState::Nothing;
-                }
-            }
-
-            FoundState::WordSpace => {
-                if token.style == Style::Highlighted {
-                    // Found "Monkey Dance"
-                    if let Some(whitespace) = previous_token {
-                        whitespace.style = Style::Highlighted;
-                    }
-
-                    found_state = FoundState::HighlightedWord;
-                } else {
-                    found_state = FoundState::Nothing;
-                }
-            }
+    for i in 1..(tokens.len() - 1) {
+        if tokens[i].token.len() == 1
+            && tokens[i].token != "\n"
+            && tokens[i].style == Style::Plain
+            && tokens[i - 1].style == Style::Highlighted
+            && tokens[i + 1].style == Style::Highlighted
+        {
+            tokens[i].style = Style::Highlighted;
         }
-
-        previous_token = Some(token);
     }
 }
 
@@ -791,4 +727,26 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn test_four_tokens_highlighting() {
+        let mut row = [
+            StyledToken::new("\n".to_string(), Style::Highlighted),
+            StyledToken::new("*".to_string(), Style::Highlighted),
+            StyledToken::new(" ".to_string(), Style::Plain),
+            StyledToken::new("Hello".to_string(), Style::Highlighted),
+        ];
+
+        bridge_consecutive_highlighted_tokens(&mut row);
+
+        assert_eq!(
+            row,
+            [
+                StyledToken::new("\n".to_string(), Style::Highlighted),
+                StyledToken::new("*".to_string(), Style::Highlighted),
+                StyledToken::new(" ".to_string(), Style::Highlighted),
+                StyledToken::new("Hello".to_string(), Style::Highlighted),
+            ]
+        );
+    }
 }
